@@ -55,6 +55,8 @@ enum
 	SECURITY_CHANGE,
 	ZOOM_CHANGE,
 	FAVICON,
+	POPUP_BLOCKED,
+	CONTEXTMENU,
 	LAST_SIGNAL
 };
 
@@ -260,6 +262,26 @@ galeon_embed_base_init (gpointer g_class)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
+	galeon_embed_signals[POPUP_BLOCKED] =
+                g_signal_new ("ge_popupblocked",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, popupblocked),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+			      G_TYPE_STRING);
+	galeon_embed_signals[CONTEXTMENU] =
+                g_signal_new ("ge_contextmenu",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, contextmenu),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__OBJECT,
+                              G_TYPE_NONE,
+                              1,
+			      GALEON_EMBED_EVENT_TYPE);
 	initialized = TRUE;
 	}
 }
