@@ -42,8 +42,8 @@
 
 #define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
 
-//#define DEBUG_MSG(x)
-#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
 
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass);
@@ -153,7 +153,10 @@ impl_shistory_go_nth (GaleonEmbed *embed,
                       int nth);
 static gboolean
 impl_shistory_copy (GaleonEmbed *source,
-                    GaleonEmbed *dest);
+		    GaleonEmbed *dest,
+		    gboolean back_history,
+		    gboolean forward_history,
+		    gboolean set_current);
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction);
@@ -226,6 +229,10 @@ mozilla_embed_security_change_cb (GtkMozEmbed *embed,
 				  gpointer request,
                                   guint state, MozillaEmbed *membed);
 
+static gint
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed);
+
 static EmbedSecurityLevel
 mozilla_embed_security_level (MozillaEmbed *membed);
 
@@ -250,6 +257,7 @@ signal_connections[] =
         { "size_to",         (void *) mozilla_embed_size_to_cb           },
         { "new_window",      (void *) mozilla_embed_new_window_cb        },
         { "security_change", (void *) mozilla_embed_security_change_cb   },
+	{ "dom_key_down",    (void *) mozilla_embed_dom_key_down_cb      },
 
         /* terminator -- must be last in the list! */
         { NULL, NULL } 
@@ -492,7 +500,7 @@ impl_can_go_up (GaleonEmbed *embed)
 	gresult result;
 
 	if (galeon_embed_get_location (embed, TRUE, FALSE, &location) != G_OK)
-	  return G_FAILED;
+		return G_FAILED;
 	g_return_val_if_fail (location != NULL, G_FAILED);
 	if ((s = mozilla_embed_get_uri_parent (location)) != NULL)
 	{
@@ -554,7 +562,8 @@ impl_go_up (GaleonEmbed *embed)
 	char *uri;
 	char *parent_uri;
 	
-	galeon_embed_get_location (embed, TRUE, FALSE, &uri);
+	if (galeon_embed_get_location (embed, TRUE, FALSE, &uri) != G_OK)
+		return G_FAILED;
 	g_return_val_if_fail (uri != NULL, G_FAILED);
 	
 	parent_uri = mozilla_embed_get_uri_parent (uri);
@@ -757,6 +766,14 @@ impl_copy_page (GaleonEmbed *dest,
 {
 	GaleonWrapper *dWrapper;
 	dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
+	if (dWrapper == NULL)
+	{
+		// We need to load some page to get the wrapper initialized.
+		// That's an implementation specific issue the caller doesn't
+		// need to know.
+		gtk_moz_embed_load_url (GTK_MOZ_EMBED(dest), "about:blank");
+		dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
+	}
 	g_return_val_if_fail (dWrapper != NULL, G_FAILED);
 
 	GaleonWrapper *sWrapper;
@@ -964,8 +981,6 @@ impl_shistory_get_nth (GaleonEmbed *embed,
                        char **aTitle)
 {
 	nsresult rv;
-        nsCAutoString url;
-	PRUnichar *title;
 	GaleonWrapper *wrapper;
 	
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
@@ -986,13 +1001,14 @@ impl_shistory_get_nth (GaleonEmbed *embed,
 		}
 	}
 	
+        nsCAutoString url;
         rv = wrapper->GetSHUrlAtIndex(nth, url);
-
         *aUrl = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
 
+	PRUnichar *title;
 	rv = wrapper->GetSHTitleAtIndex(nth, &title);
-
 	*aTitle = g_strdup (NS_ConvertUCS2toUTF8(title).get());
+	nsMemory::Free(title);
 
 	return G_OK;
 }
@@ -1032,7 +1048,10 @@ impl_shistory_go_nth (GaleonEmbed *embed,
 
 static gboolean
 impl_shistory_copy (GaleonEmbed *source,
-                    GaleonEmbed *dest)
+		    GaleonEmbed *dest,
+		    gboolean back_history,
+		    gboolean forward_history,
+		    gboolean set_current)
 {
 	nsresult rv;
 	GaleonWrapper *s_wrapper;
@@ -1044,7 +1063,7 @@ impl_shistory_copy (GaleonEmbed *source,
 	d_wrapper = MOZILLA_EMBED(dest)->priv->wrapper;
 	g_return_val_if_fail (d_wrapper != NULL, G_FAILED);
 
-	rv = s_wrapper->CopyHistoryTo (d_wrapper);
+	rv = s_wrapper->CopyHistoryTo (d_wrapper, back_history, forward_history, set_current);
 
 	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
@@ -1118,13 +1137,7 @@ impl_get_security_level (GaleonEmbed *embed,
 
 		if (tooltip)
 		{
-			const nsString &string = nsString(tooltip);
-			char *tmp;
-			tmp = ToNewCString (string);
-			
-			*description = g_strdup (tmp);
-
-			nsMemory::Free (tmp);
+			*description = g_strdup (NS_ConvertUCS2toUTF8(tooltip).get());
 		}
 	}
 	
@@ -1223,20 +1236,20 @@ static gresult
 impl_set_charset (GaleonEmbed *embed,
                   const char *charset)
 {
-	nsresult result = NS_OK;
+	nsresult rv;
 	GaleonWrapper *wrapper;
-	char *cset;
 
-	cset = g_strdup (charset);
-	
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
-	wrapper->ForceCharacterSet (cset);
-
-	g_free (cset);
+	rv = wrapper->ForceCharacterSet (charset);
+	if (NS_SUCCEEDED (rv))
+	{
+		gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
+				      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
+	}
 	
-	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
+	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
 
 static void
@@ -1339,7 +1352,7 @@ mozilla_embed_link_message_cb (GtkMozEmbed *embed,
 	*getter_Copies(message) = gtk_moz_embed_get_link_message_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_link_message",
-			       g_strdup(NS_ConvertUCS2toUTF8(message).get()));
+			       NS_ConvertUCS2toUTF8(message).get());
 }
 
 static void
@@ -1351,7 +1364,7 @@ mozilla_embed_js_status_cb (GtkMozEmbed *embed,
 	*getter_Copies(status) = gtk_moz_embed_get_js_status_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_js_status",
-			       g_strdup(NS_ConvertUCS2toUTF8(status).get()));
+			       NS_ConvertUCS2toUTF8(status).get());
 }
 
 static void
@@ -1360,14 +1373,14 @@ mozilla_embed_visibility_cb (GtkMozEmbed *embed, gboolean visibility,
 {
 	g_signal_emit_by_name (membed, "ge_visibility", visibility); 
 
+	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	if (wrapper == NULL) return;
+
 	nsresult rv;
 	nsCOMPtr<nsIWindowWatcher> wwatch
 		(do_GetService(WINDOWWATCHER_CONTRACTID, &rv));
 	if (NS_FAILED(rv) || !wwatch) return;
 
-	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_if_fail (wrapper != NULL);
-
 	nsCOMPtr<nsIDOMWindow> domWindow;
 	rv = wrapper->GetDOMWindow(getter_AddRefs(domWindow));
 	if(NS_FAILED(rv) || !domWindow) return;
@@ -1387,31 +1400,36 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 				  MozillaEmbed *membed)
 {
 	GaleonEmbedEvent *info;
-	EventContext event_context;
 	gint return_value = 0;
 	GaleonWrapper *wrapper;
 	nsresult result;
 
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	g_return_val_if_fail(wrapper != NULL, G_FAILED);
 	
-	info = galeon_embed_event_new ();
+	info = galeon_embed_event_new();
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
+	EventContext ctx;
+	ctx.Init(wrapper);
+        result = ctx.GetMouseEventInfo(static_cast<nsIDOMMouseEvent*>(dom_event), info);
+	if (NS_SUCCEEDED(result))
+	{
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		result = ctx.GetTargetDocument(getter_AddRefs(domDoc));
+		if (NS_SUCCEEDED(result))
+		{
+			result = wrapper->PushTargetDocument(domDoc);
+			if (NS_SUCCEEDED(result))
+			{
+				g_signal_emit_by_name(membed, "ge_dom_mouse_click", 
+						      info, &return_value); 
+				wrapper->PopTargetDocument();
+			}
+		}
 
-        nsCOMPtr<nsIDOMDocument> domDoc;
-        result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
-        if (NS_SUCCEEDED(result))
-        {
-                result = wrapper->PushTargetDocument (domDoc);
-        }
+	}
 
-	g_signal_emit_by_name (membed, "ge_dom_mouse_click", info, &return_value); 
-	
-	wrapper->PopTargetDocument ();
-	
-	g_object_unref (info);
+	g_object_unref(info);
 	
 	return return_value;
 }
@@ -1421,7 +1439,6 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 				 MozillaEmbed *membed)
 {
 	GaleonEmbedEvent *info;
-	EventContext event_context;
 	gint return_value = 0;
 	GaleonWrapper *wrapper;
 	nsresult result;
@@ -1431,12 +1448,13 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	
 	info = galeon_embed_event_new ();
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
+	EventContext ctx;
+	ctx.Init (wrapper);
+        result = ctx.GetMouseEventInfo (static_cast<nsIDOMMouseEvent*>(dom_event), info);
 	if (NS_SUCCEEDED(result))
 	{
 		nsCOMPtr<nsIDOMDocument> domDoc;
-		result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
+		result = ctx.GetTargetDocument (getter_AddRefs(domDoc));
 		if (NS_SUCCEEDED(result))
 		{
 			result = wrapper->PushTargetDocument (domDoc);
@@ -1455,6 +1473,66 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	return return_value;
 }
 
+static gint
+mozilla_embed_dom_key_down_cb (GtkMozEmbed *embed, gpointer dom_event,
+		               MozillaEmbed *membed)
+{
+	nsCOMPtr<nsIDOMKeyEvent> ev = static_cast<nsIDOMKeyEvent*>(dom_event);
+	if (!ev)
+	{
+		return FALSE;
+	}
+
+	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+
+	GaleonEmbedEvent *info;
+	info = galeon_embed_event_new ();
+
+	gboolean ret = FALSE;
+
+	// Just check for Shift-F10 so that we know to popup the context menu.
+	//
+	// The DOM_VK_* keycodes are not compatible with the keycodes defined
+	// in GDK, so making a generic dom_key_down signal is probably not
+	// worth the trouble.
+
+	nsresult rv;
+	EventContext ctx;
+	ctx.Init (wrapper);
+	rv = ctx.GetKeyEventInfo (ev, info);
+	if (NS_SUCCEEDED(rv) &&
+	    (info->keycode == nsIDOMKeyEvent::DOM_VK_F10 &&
+	     info->modifier == GDK_SHIFT_MASK))
+	{
+		// Translate relative coordinates to absolute values, and try
+		// to avoid covering links by adding a little offset.
+
+		int x, y;
+		gdk_window_get_origin (GTK_WIDGET(membed)->window, &x, &y);
+		info->x += x + 6;	
+		info->y += y + 6;
+
+		nsCOMPtr<nsIDOMDocument> doc;
+		rv = ctx.GetTargetDocument (getter_AddRefs(doc));
+		if (NS_SUCCEEDED(rv))
+		{
+			rv = wrapper->PushTargetDocument (doc);
+			if (NS_SUCCEEDED(rv))
+			{
+				g_signal_emit_by_name (membed, "ge_contextmenu", info);
+				wrapper->PopTargetDocument ();
+
+				// return TRUE to stop further event handling
+				ret = TRUE;
+			}
+		}
+	}
+
+	g_object_unref (info);
+	return ret;
+}
+
 static void
 mozilla_embed_size_to_cb (GtkMozEmbed *embed, gint width, gint height, 
 			  MozillaEmbed *membed)
