@@ -62,7 +62,7 @@ typedef struct {
 	guint open_in_new_tab : 1;
 } GaleonAutomationLoadurlData;
 
-GSList *postponed_loadurls = NULL;
+static GSList *postponed_loadurls = NULL;
 static gboolean initialized = FALSE;
 
 static BonoboObject *
@@ -168,11 +168,25 @@ galeon_automation_complete_initialization (void)
 	/* no window open, let's try to autoresume */
 	if (session_get_windows (session) == NULL)
 	{
-		gboolean res;
+		SessionResumeType res;
 		res = session_autoresume (session);
+
+		/* Don't load any windows, the use wants to abort this
+		 * instance */
+		if ( res == SESSION_ABORT ) {
+			GSList * iterator;
+			for( iterator = postponed_loadurls ; iterator ; iterator = iterator->next) {
+				GaleonAutomationLoadurlData *data = iterator->data;
+				g_free (data->url);
+				g_free (data);
+			}
+			g_slist_free (postponed_loadurls);
+			return;
+		}
+		
 		/* no need to open the homepage,
 		 * we did already open session windows */
-		if (res && postponed_loadurls) {
+		if (res == SESSION_RESUMED && postponed_loadurls) {
 			GSList *last = g_slist_last (postponed_loadurls);
 			GaleonAutomationLoadurlData *data = last->data;
 			if (data->url == NULL || *data->url == 0) {
