@@ -39,8 +39,8 @@ static gboolean
 galeon_main_automation_init (void);
 static gint
 galeon_main_translate_url_arguments (poptContext context, gchar ***urls);
-static gboolean
-galeon_main_start (gpointer data);
+static void
+galeon_main_start (void);
 
 /* FIXME */
 GnomeProgram *program;
@@ -61,7 +61,6 @@ static gboolean open_as_nautilus_view = FALSE;
 static BonoboObject *automation_object;
 static gint n_urls; 
 static gchar **url;
-static gboolean first_instance;
 
 /* command line argument parsing structure */
 static struct poptOption popt_options[] =
@@ -111,6 +110,13 @@ static struct poptOption popt_options[] =
 	{ NULL, 0, 0, NULL, 0, NULL, NULL }
 };
 
+static gboolean
+unref_when_idle (gpointer user_data)
+{
+	g_object_unref (user_data);
+	return FALSE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -120,10 +126,13 @@ main (int argc, char *argv[])
 	
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
-	bindtextdomain(GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
+	bindtextdomain(GETTEXT_PACKAGE, DATADIR "/locale");
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 #endif
+	/* set the application name manually until gnome handles the
+	 * GNOME_PARAM_HUMAN_READABLE_NAME in a sane fashion */
+	g_set_application_name (_("Galeon"));
 
 	program = gnome_program_init (PACKAGE, VERSION,
                                       LIBGNOMEUI_MODULE, argc, argv,
@@ -141,30 +150,43 @@ main (int argc, char *argv[])
 	/* load arguments that aren't regular options (urls to load) */
         n_urls = galeon_main_translate_url_arguments (context, &url);
 
-	first_instance = galeon_main_automation_init ();
-
-	if (first_instance)
+	if (galeon_main_automation_init ())
 	{
 		gnome_vfs_init ();
 		
 		glade_gnome_init ();
 		
-		gnome_window_icon_set_default_from_file (PIXMAP_DIR "/galeon.png");
+		gnome_window_icon_set_default_from_file (DATADIR "/pixmaps/galeon.png");
 		
 		galeon_shell_new ();
 		
-		g_idle_add ((GSourceFunc) galeon_main_start, NULL);
+		galeon_main_start ();
+
+		/* Flush any pending GTK events, this works around crashes when
+		 * it takes a long time to startup. */
+		while (gtk_events_pending())
+			gtk_main_iteration();
+		
+		galeon_automation_complete_initialization ();
+		
+		g_idle_add (unref_when_idle, galeon_shell);
 		
 		bonobo_main ();
 		
 		gnome_vfs_shutdown ();
+	} 
+	else
+	{
+#if GTK_CHECK_VERSION(2,2,0)
+		gdk_notify_startup_complete ();
+#endif 
 	}
 
 	return 0;
 }
 
-static gboolean
-galeon_main_start (gpointer data)
+static void
+galeon_main_start (void)
 {
 	GNOME_GaleonAutomation gaserver;
 	int i;
@@ -174,6 +196,14 @@ galeon_main_start (gpointer data)
 	gaserver = bonobo_activation_activate_from_id ("OAFIID:GNOME_Galeon_Automation",
 						       0, NULL, &corba_env);
 
+	/* if none of the open in tab/window/existing options is given, fall
+	 * back to open in a new window, otherwise the url can get opened in a
+	 * different desktop without the user realizing it */
+	if (!open_in_existing && !open_in_new_window && !open_in_new_tab)
+	{
+		open_in_new_window = TRUE;
+	}
+
 	if (gaserver == NULL)
 	{
 		GtkWidget *dialog;
@@ -248,23 +278,12 @@ galeon_main_start (gpointer data)
 		}
 	}
 
-	/* Unref so it will exit if no more used */
-	if (first_instance)
-	{
-		g_object_unref (G_OBJECT(galeon_shell));
-	}
-
 	if (gaserver)
 	{
 		bonobo_object_release_unref (gaserver, &corba_env);
 	}
 
 	CORBA_exception_free (&corba_env);
-
-#if GTK_CHECK_VERSION(2,2,0)
-	gdk_notify_startup_complete ();
-#endif 
-	return FALSE;
 }
 
 static gboolean
@@ -284,7 +303,7 @@ galeon_main_automation_init (void)
 	}
 	else
 	{
-		galeon_main_start (NULL);
+		galeon_main_start ();
 		g_message (_("Galeon already running, using existing process"));
 		return FALSE;
 	}
