@@ -27,12 +27,20 @@
 #include "gul-gui.h"
 #include "gul-string.h"
 #include "galeon-navigation-button.h"
+#include "galeon-shell.h"
 #include <gtk/gtkstock.h>
 #include <string.h>
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
-#define DEBUG_MSG(x) g_print x
-//#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
+
+/* these should be kept in sync:
+ *   bookmarks/bookmarks-gtk-menu-item.h   - GB_GTK_MENU_ITEM_NAME_MAX_LENGTH
+ *   src/galeon-navigation-button.c        - MENU_ITEM_MAX_LENGTH
+ *   src/window-recent-history-menu.c      - MENU_ITEM_MAX_LENGTH
+ */
+#define MENU_ITEM_MAX_LENGTH 40
 
 /**
  * Private data
@@ -437,12 +445,26 @@ galeon_navigation_button_clicked_cb (GtkWidget *w, GaleonNavigationButton *b)
 /* TODO: clean all this, it came from toolbar.c and is messy */
 
 static GtkWidget *
-new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, const GdkPixbuf *ico)
+new_num_accel_menu_item (gint num, const gchar *origtext, gboolean lettersok, const GdkPixbuf *ico)
 {
-        gchar *text = gul_string_new_num_accel (num, origtext, lettersok);
-        GtkWidget *item = gtk_image_menu_item_new ();
-        GtkWidget *hb = gtk_hbox_new (FALSE, 0);
-        GtkWidget *label = gtk_label_new_with_mnemonic (text);
+        gchar *shortened, *escaped, *text;
+        GtkWidget *item;
+        GtkWidget *hb;
+        GtkWidget *label;
+
+	/* shorten, escape, add accel */
+	shortened = gul_string_shorten (origtext, MENU_ITEM_MAX_LENGTH);
+	escaped = gul_string_escape_uline_accel (shortened);
+
+	text = gul_string_new_num_accel (num, escaped, lettersok);
+	g_free (shortened);
+	g_free (escaped);
+
+	if (text == NULL) return NULL;
+
+	item = gtk_image_menu_item_new ();
+	hb = gtk_hbox_new (FALSE, 0);
+	label = gtk_label_new_with_mnemonic (text);
 
         gtk_box_pack_start (GTK_BOX (hb), label, FALSE, FALSE, 0);
         gtk_container_add (GTK_CONTAINER (item), hb);
@@ -452,6 +474,8 @@ new_num_accel_menu_item (gint num, gchar *origtext, gboolean lettersok, const Gd
 
         gtk_widget_show_all (item);
 
+	g_free (text);
+
         return item;
 }
 
@@ -469,6 +493,42 @@ activate_back_or_forward_menu_item_cb (GtkWidget *menu, GaleonWindow *window)
 	galeon_embed_shistory_go_nth (embed, go_nth);
 }
 
+static gboolean
+button_press_event_back_or_forward_menu_item_cb  (GtkWidget *widget, GdkEventButton *event, 
+						  GaleonWindow *window)
+{
+	if (event->button == 2 && IS_GALEON_WINDOW (window))
+	{
+		GaleonEmbed *embed;
+		int go_nth;
+		gchar *url;
+		gchar *title;
+
+		embed = galeon_window_get_active_embed (window);
+		g_return_val_if_fail (embed != NULL, FALSE);
+		
+		go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "go_nth"));
+		
+		if (galeon_embed_shistory_get_nth (embed, go_nth, FALSE, &url, &title) == G_OK)
+		{
+			g_free (title);
+
+			if (url)
+			{
+				GaleonTab *tab = galeon_window_get_active_tab (window);
+				galeon_shell_new_tab (galeon_shell, window, tab, url, 
+						      GALEON_NEW_TAB_JUMP);
+				g_free (url);
+			}
+		}
+		return TRUE;
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+
 static void
 activate_up_menu_item_cb (GtkWidget *menu, GaleonWindow *window)
 {
@@ -494,6 +554,43 @@ activate_up_menu_item_cb (GtkWidget *menu, GaleonWindow *window)
 	g_slist_free (l);
 }
 
+static gboolean
+button_press_event_up_menu_item_cb  (GtkWidget *widget, GdkEventButton *event, 
+				     GaleonWindow *window)
+{
+	if (event->button == 2 && IS_GALEON_WINDOW (window))
+	{
+		GaleonEmbed *embed;
+		int go_nth;
+		gchar *url;
+		GSList *l;
+
+		embed = galeon_window_get_active_embed (window);
+		g_return_val_if_fail (embed != NULL, FALSE);
+		
+		go_nth = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "go_nth"));
+		
+		galeon_embed_get_go_up_list (embed, &l);
+		url = g_slist_nth_data (l, go_nth);
+
+		if (url)
+		{
+			GaleonTab *tab = galeon_window_get_active_tab (window);
+			galeon_shell_new_tab (galeon_shell, window, tab, url, 
+					      GALEON_NEW_TAB_JUMP);
+		}
+
+		g_slist_foreach (l, (GFunc) g_free, NULL);
+		g_slist_free (l);
+
+		return TRUE;
+	}
+	else
+	{
+		return FALSE;
+	}
+}
+
 static void
 setup_back_or_forward_menu (GaleonWindow *window, GtkMenuShell *ms, GaleonNavigationDirection dir)
 {
@@ -522,21 +619,25 @@ setup_back_or_forward_menu (GaleonWindow *window, GtkMenuShell *ms, GaleonNaviga
 
 	while (start != end)
 	{
-		char *title, *url, *tmp;
+		char *title, *url;
 		GtkWidget *item;
+
 		galeon_embed_shistory_get_nth (embed, start, FALSE, &url, &title);
-		tmp = gul_string_escape_uline_accel (title);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, NULL);
+		g_free (url);
+
+		item = new_num_accel_menu_item (accell_count, title, TRUE, NULL);
+		g_free (title);
+
+		if (item == NULL) break;
+
         	gtk_menu_shell_append (ms, item);
 		g_object_set_data (G_OBJECT (item), "go_nth", GINT_TO_POINTER (start));
 		g_signal_connect (item, "activate",
                                   G_CALLBACK (activate_back_or_forward_menu_item_cb), window);
+		g_signal_connect (item, "button-press-event",
+                                  G_CALLBACK (button_press_event_back_or_forward_menu_item_cb), window);
 		gtk_widget_show_all (item);
 
-		g_free (url);
-		g_free (title);
-		g_free (tmp);
-
 		accell_count++;
 		if (start < end)
 		{
@@ -565,18 +666,20 @@ setup_up_menu (GaleonWindow *window, GtkMenuShell *ms)
 	for (li = l; li; li = li->next)
 	{
 		char *url = li->data;
-		char *tmp;
 		GtkWidget *item;
-		tmp = gul_string_escape_uline_accel (url);
-		item = new_num_accel_menu_item (accell_count, tmp, TRUE, NULL);
+
+		item = new_num_accel_menu_item (accell_count, url, TRUE, NULL);
+		if (item == NULL) break;
+
         	gtk_menu_shell_append (ms, item);
 		g_object_set_data (G_OBJECT(item), "go_nth", GINT_TO_POINTER (accell_count));
 		g_signal_connect (item, "activate",
                                   G_CALLBACK (activate_up_menu_item_cb), window);
+		g_signal_connect (item, "button-press-event",
+                                  G_CALLBACK (button_press_event_up_menu_item_cb), window);
 		gtk_widget_show_all (item);
 
 		accell_count++;
-		g_free (tmp);
 	}
 	
 	g_slist_foreach (l, (GFunc) g_free, NULL);
