@@ -184,6 +184,7 @@ static BonoboUIVerb galeon_popup_verbs [] = {
 
 struct GaleonWindowPrivate
 {
+	GList *notifiers;
 	PPViewToolbar *ppview_toolbar;
 	Toolbar *toolbar;
 	Statusbar *statusbar;
@@ -499,9 +500,9 @@ setup_notebook (GaleonWindow *window)
 			  G_CALLBACK (galeon_window_tab_detached_cb), 
 			  window);
 
-	eel_gconf_notification_add(CONF_TABS_TABBED_EDGE,
-				   (GConfClientNotifyFunc)tabbed_position_gconf_changed_cb,
-				   notebook);
+	galeon_notification_add(CONF_TABS_TABBED_EDGE,
+				(GConfClientNotifyFunc)tabbed_position_gconf_changed_cb,
+				notebook, &window->priv->notifiers);
     
 	gtk_widget_show (GTK_WIDGET (notebook));
 
@@ -826,9 +827,9 @@ setup_web_menu (GaleonWindow *window)
 					 accept_cookies_bonoboui_changed_cb,
 					 GINT_TO_POINTER(ACCEPT_COOKIES_NEVER));
 
-	eel_gconf_notification_add(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
-				   (GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
-				   ui_component);
+	galeon_notification_add(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
+				(GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
+				ui_component, &window->priv->notifiers);
 
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
@@ -850,18 +851,18 @@ setup_web_menu (GaleonWindow *window)
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_JAVA_ENABLED);
 
-	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
-				   (GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
-				   window);
-	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
-	eel_gconf_notification_add(CONF_FILTERING_JAVASCRIPT_ENABLED,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
-	eel_gconf_notification_add(CONF_FILTERING_JAVA_ENABLED,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
+	galeon_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				(GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
+				window, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_JAVASCRIPT_ENABLED,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_JAVA_ENABLED,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
 }
 
 static void
@@ -910,9 +911,9 @@ setup_images_menu(GaleonWindow *window)
 					 view_images_bonoboui_changed_cb,
 					 GINT_TO_POINTER(VIEW_IMAGES_NEVER));
 
-	eel_gconf_notification_add(CONF_FILTERING_IMAGE_LOADING_TYPE, 
-				   (GConfClientNotifyFunc)view_images_gconf_changed_cb, 
-				   ui_component);
+	galeon_notification_add(CONF_FILTERING_IMAGE_LOADING_TYPE, 
+				(GConfClientNotifyFunc)view_images_gconf_changed_cb, 
+				ui_component, &window->priv->notifiers);
 }
 
 static void
@@ -1136,6 +1137,8 @@ galeon_window_finalize (GObject *object)
 
         g_return_if_fail (window->priv != NULL);
 
+	galeon_notification_remove (&window->priv->notifiers);
+
 	remove_from_session (window);
 
 	if (window->priv->embed_popup)
@@ -1146,6 +1149,8 @@ galeon_window_finalize (GObject *object)
 	g_object_unref (G_OBJECT (window->priv->bookmarks_menu));
 	g_object_unref (G_OBJECT (window->priv->bookmarks_context_menu));
 	g_object_unref (G_OBJECT (window->priv->bookmarks_toolbar));
+	/* break circular references in the toolbar */
+	g_object_run_dispose (G_OBJECT(window->priv->toolbar));
 	g_object_unref (G_OBJECT (window->priv->toolbar));
 	g_object_unref (G_OBJECT (window->priv->statusbar));
 	
@@ -1172,6 +1177,8 @@ galeon_window_finalize (GObject *object)
 	g_object_unref (window->priv->recent_history);
 	g_object_unref (window->priv->recent_history_menu);
 
+	bonobo_object_unref (window->ui_component);
+
         g_free (window->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
