@@ -73,6 +73,7 @@
 
 static void toolbar_class_init (ToolbarClass *klass);
 static void toolbar_init (Toolbar *t);
+static void toolbar_dispose (GObject *object);
 static void toolbar_finalize (GObject *object);
 static void toolbar_set_window (Toolbar *t, GaleonWindow *window);
 static void toolbar_get_widgets (Toolbar *t);
@@ -113,6 +114,8 @@ struct ToolbarPrivate
 	GtkWidget *spinner;
 	gboolean visibility;
 
+	guint style_notification_id;
+
 	GSList *navigation_buttons;
 	GaleonReloadButton *reload_button;
 	GaleonGoButton *go_button;
@@ -165,6 +168,7 @@ toolbar_class_init (ToolbarClass *klass)
 
         parent_class = g_type_class_peek_parent (klass);
 
+        object_class->dispose = toolbar_dispose;
         object_class->finalize = toolbar_finalize;
 
 	object_class->set_property = toolbar_set_property;
@@ -422,8 +426,9 @@ toolbar_set_window (Toolbar *t, GaleonWindow *window)
 
 	gul_tb_bonobo_view_set_path (t->priv->bview, t->priv->ui_component, "/Toolbar");
 
-	eel_gconf_notification_add (CONF_TOOLBAR_STYLE,
-			            toolbar_style_changed_cb, t);
+	t->priv->style_notification_id = 
+		eel_gconf_notification_add (CONF_TOOLBAR_STYLE,
+				            toolbar_style_changed_cb, t);
 
 	toolbar_setup_style (t);
 	toolbar_get_widgets (t);
@@ -743,6 +748,22 @@ toolbar_changed_cb (GulToolbar *gt, Toolbar *t)
 	}
 }
 
+static void
+toolbar_dispose (GObject *object)
+{
+	Toolbar *t = (Toolbar *) object;
+
+        g_return_if_fail (IS_TOOLBAR (object));
+
+	if (t->priv->bview)
+	{
+		g_object_unref (t->priv->bview);
+		g_object_unref (t->priv->ui_component);
+		t->priv->bview = NULL;
+		t->priv->ui_component = NULL;
+	}
+}
+
 static void
 toolbar_finalize (GObject *object)
 {
@@ -758,6 +779,8 @@ toolbar_finalize (GObject *object)
 
         g_return_if_fail (p != NULL);
 
+	eel_gconf_notification_remove (p->style_notification_id);
+
 	if (p->location_entry) g_object_unref (p->location_entry);
 	if (p->favicon_ebox) g_object_unref (p->favicon_ebox);
 	if (p->favicon) g_object_unref (p->favicon);
@@ -769,8 +792,6 @@ toolbar_finalize (GObject *object)
 		g_source_remove (p->zoom_timeout_id);
 	}
 
-	g_object_unref (t->priv->bview);
-	
 	for (li = t->priv->navigation_buttons; li; li = li->next)
 	{
 		g_object_unref (li->data);
