@@ -22,6 +22,9 @@
  * code in there if possible, so other apps like gedit or profterm can use 
  * it easily if they want.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gul-notebook.h"
 #include "gul-gui.h"
